     /********************************************************************
      *                                                                  *
      *         Copyright (c) 1985 by                                    *
      *            Lewis Makepeace and Stan Zanarotti                    *
      *                                                                  *
      *                                                                  *
      *         All rights reserved.                                     *
      *                                                                  *
      ********************************************************************/

/*
 * interface.h -- Include file for core interface.  This file contains
 * definitions for structures and routines that will be used to communicate
 * with the core discuss routines.  The exact protocol between the two 
 * has not been defined, although something ala RPC should be investiaged
 */

#ifndef INTERFACE_H_INCLUDED
#define INTERFACE_H_INCLUDED

#include <stdio.h>
#include <discuss/types.h>
#include <discuss/tfile.h>
#include <discuss/dsname.h>
#include <discuss/acl.h>
#include <discuss/dsname.h>

/* this include maybe should be split apart eventually. */

/* Derived from CORE.PAS 06/21/86 by SRZ */


/*
 *
 *   trn_info: This is the information about a transaction that is
 *   returned by core routines, except the transaction contents itself.
 *   This is for list commands, etc.
 *
 */

typedef struct {
	int version;		 /* version of this structure */
	trn_nums current;	 /* trans # of this transaction */
	trn_nums prev;		 /* trans # of previous non-deleted trans */
	trn_nums next;		 /* etc, etc. */
	trn_nums pref;
	trn_nums nref;
	trn_nums fref;
	trn_nums lref;

 	int chain_index;	 /* index of transaction in chain. Fref is 1 */
	time_t date_entered;	 /* date/time transaction was entered */
	int num_lines;		 /* # lines in transaction */
	int num_chars;		 /* # chars in transaction */
	char *subject;		 /* subject of transaction */
	char *author;		 /* author of transaction */
} trn_info;

/* Same as above, except includes flags */

typedef struct {
	int version;		 /* version of this structure */
	trn_nums current;	 /* trans # of this transaction */
	trn_nums prev;		 /* trans # of previous non-deleted trans */
	trn_nums next;		 /* etc, etc. */
	trn_nums pref;
	trn_nums nref;
	trn_nums fref;
	trn_nums lref;

 	int chain_index;	 /* index of transaction in chain. Fref is 1 */
	time_t date_entered;	 /* date/time transaction was entered */
	int num_lines;		 /* # lines in transaction */
	int num_chars;		 /* # chars in transaction */
	char *subject;		 /* subject of transaction */
	char *author;		 /* author of transaction */
	int flags;		 /* Miscellaneous flags */
} trn_info2;

/* Same as above, except includes signature */

typedef struct {
	int version;		 /* version of this structure */
	trn_nums current;	 /* trans # of this transaction */
	trn_nums prev;		 /* trans # of previous non-deleted trans */
	trn_nums next;		 /* etc, etc. */
	trn_nums pref;
	trn_nums nref;
	trn_nums fref;
	trn_nums lref;

 	int chain_index;	 /* index of transaction in chain. Fref is 1 */
	time_t date_entered;	 /* date/time transaction was entered */
	int num_lines;		 /* # lines in transaction */
	int num_chars;		 /* # chars in transaction */
	char *subject;		 /* subject of transaction */
	char *author;		 /* author of transaction */
	int flags;		 /* Miscellaneous flags */
	char *signature;	 /* signature (soft author) of transaction */
} trn_info3;

/* Meanings of flags */
#define TRN_FDELETED 1
#define TRN_FLAG1    2

/*
 *
 * mtg_info:  This is information about a meeting that is returned
 *   by core routines, giving information about the meeting itself.
 *
 */

typedef struct {
	int version;			/* version of this structure */
	pathnames location;		/* location of actual meeting */
	mtg_names long_name;		/* long name of meeting */
	char *chairman;			/* #1 chairman of meeting */
	trn_nums first,last;		/* first & last non-dl trns of mtg */
	trn_nums lowest,highest;	/* first & last maybe-dl trns */

	time_t date_created;		/* date meeting created */
	time_t date_modified;		/* last modification of meeting */

	bool public_flag;		/* if mtg is public */
	char *access_modes;		/* Access modes we have to this mtg. */
} mtg_info;


/*
 *
 * Routines for interface.
 *
 */

void add_trn (char *mtg_name, tfile source_file, char *subject,
	      trn_nums reply_trn, trn_nums *result_trn, int *result);
/*
 *
 * adds a transaction to the current meeting, either as a reply or an
 * original transaction.  Returns an error code, and the transaction number
 * given to the transaction
 *
 */

void add_trn2 (char *mtg_name, tfile source_file, char *subject, char *s,
	       trn_nums reply_trn, trn_nums *result_t, int *result);

/*
 *
 * adds a transaction to the current meeting, either as a reply or an
 * original transaction.  Allows a signature (soft author) to be added.
 * Returns an error code, and the transaction number given to the transaction
 *
 */

void get_trn_info (char *mtg_name, trn_nums trn, trn_info *info, int *result);

/*
 *
 * returns information about the given transaction in info, with an error
 * code as its return argument
 *
 */

void get_trn_info2 (char *mtg_name, trn_nums trn, trn_info2 *info, int *);

/*
 *
 * returns some more information about the given transaction in info, with an
 * error code as its return argument
 *
 */

void get_trn_info3 (char *mtg_name, trn_nums trn, trn_info3 *info, int *);

/*
 *
 * returns some more information about the given transaction in info, with an
 * error code as its return argument
 *
 */

void delete_trn (char *mtg_name, trn_nums trn, int *result);

/*
 *
 * deletes the given transaction from the current meeting.  Returns an
 * error code
 *
 */

void retrieve_trn (char *mtg_name, trn_nums trn, int *result);

/*
 *
 * retrieves a previously deleted transaction from the current meeting, if
 * possible.  trn must refer to a deleted transaction.  An error code is
 * returned
 *
 */

void create_mtg (char *location, char *long_mtg_name, bool public, int *);

/*
 *
 * Creates a new meeting with the given long_mtg name, where location is the
 * it's place in the hierarchy, and the long_mtg_name is its canonical name.
 * The chairman of the new meeting is the current user.
 *
 */

void get_mtg_info (char *mtg_name, mtg_info *info, int *result);

/*
 *
 * returns information about the given meeting.  Return argument is an
 * error code
 *
 */

void start_mtg_info (void);

void get_trn (char *mtg_name, trn_nums trn, tfile dest_file, int *result);

/*
 *
 * gets the given transaction, and feeds it through dest_file.  Returns an
 * error code
 *
 */

void remove_mtg (char *mtg_name, int *result);

/*
 *
 * removes the given meeting from the meeting list.  This should
 * probably go ahead and physically remove the contents of the meeting
 * but that's not obvious if we have the access to do that.
 *
 */


/* announce.c */
void dsc_announce_mtg (name_blk *, name_blk *, int, tfile, int *, int *);

/* auth.c */
int get_authenticator (char *, int, char **, int *, int *);

/* conv_mgr.c */
char *get_module (void);
void set_module (char *, int *, int *);
void flush_convs (void);
int get_conv_server_version (void);

/* dsc_enter.c */
int dsc_enter (FILE *, char *, char *, int *);

/* host.c */
char *local_host_name (void);
int namcmp (char *, char *);

/* interface.c */
int dsc_add_trn (name_blk *, tfile, char *, trn_nums, trn_nums *, int *);
int dsc_add_trn2 (name_blk *, tfile, char *, char *, trn_nums, trn_nums *, int *);
int dsc_get_trn_info (name_blk *, trn_nums, trn_info *, int *);
int dsc_get_trn_info2 (name_blk *, trn_nums, trn_info2 *, int *);
int dsc_get_trn_info3 (name_blk *, trn_nums, trn_info3 *, int *);
int dsc_set_trn_flags (name_blk *, trn_nums, int, int *);
int dsc_delete_trn (name_blk *, trn_nums, int *);
int dsc_retrieve_trn (name_blk *, trn_nums, int *);
int dsc_create_mtg (char *, char *, char *, bool, bool , int *);
int dsc_get_mtg_info (name_blk *, mtg_info *, int *);
int dsc_get_trn (name_blk *, trn_nums, tfile, int *);
int dsc_remove_mtg (name_blk *, int *);
int dsc_updated_mtg (name_blk *, bool *, int *);
int dsc_get_acl (name_blk *, int *, dsc_acl **);
int dsc_get_access (name_blk *, char *, char **, int *);
int dsc_set_access (name_blk *, char *, char *, int *);
int dsc_delete_access (name_blk *, char *, int *);
int dsc_whoami (name_blk *, char **, int *);
void dsc_destroy_mtg_info (mtg_info *);
void dsc_destroy_trn_info (trn_info *);
void dsc_destroy_trn_info3 (trn_info3 *);
int log_warning (int, char *);
int set_warning_hook (int (*hook)(int, char *));

/* realm.c */
const char *local_realm (void);

/* res_module.c */
void resolve_module (char *, int *, char **, char **, int *);


/* rpcall.c */
void startsend (int);
void sendint (int);
void sendshort (int);
void sendstr (char *);
void sendbool (int);
void sendfile (tfile);
void sendit (char *);
void init_rpc (void);
void term_rpc (void);
void recvreply (void);
int recvint (void);
int recvshort (void);
char *recvstr (void);
int recvbool (void);
void senddata (tfile);
void recvdata (tfile);

/* stubs.c */
void updated_mtg (char *, int, int , bool *, int *);
void get_acl (char *, int *, dsc_acl **);
void get_access (char *, char *, char **, int *);
void set_access (char *, char *, char *, int *);
void delete_access (char *, char *, int *);
void set_trn_flags (char *, trn_nums, int, int *);
void dwhoami (char **, int *);
void recv_trn_info (trn_info *);
void recv_trn_info2 (trn_info2 *);
void recv_trn_info3 (trn_info3 *);
void recv_mtg_info (mtg_info *);
int get_server_version (void);

/* tfpager.c */
void set_no_echo (void);
void set_echo (void);
int crank (void);
int init_tfpager (void);
tfile pager_tfile (void);

/* tmem.c */
tfile mem_tfile (char *, int);

/* tunix.c */
int tunix (int, char **, int *, char *, int , int *);
tfile unix_tfile (int);

#endif
